#!/usr/bin/env python
# p4submitters.py.py

import string
import os
import time
import UserDict

_now = time.time ()
_halfYear = 365 / 2 * 24 * 60 * 60

import vmake.yutil

class Bag (UserDict.UserDict):
    def add (self, key, count = 1):
        self[key] = self.get (key, 0) + count

def readTelFile ():
    telFile = os.path.join (os.environ ['TOOL'], 'bin', 'tel.pl')
    stream = open (telFile, 'r')
    line = stream.readline ()
    while line != '__DATA__\n':
        line = stream.readline ()
    result = {}
    for line in stream.readlines ():
        fields = map (string.strip, string.split (line, ","))
        result [string.lower (fields [2])] = fields [0]
    stream.close ()
    return result
        
_duserLookup = readTelFile ()
        
def timeWeigth (dateString):
    year, month, day = map (int, string.split (dateString, '/'))
    then = time.mktime (year, month, day, 0, 0, 0, 0, 0, -1)
    diff = _now - then
    if diff < _halfYear:
        result = 4
    elif diff < 4 * _halfYear:
        result = 2
    else:
        result = 1
    return result

def p4submitters (timeWeighted, orderWeighted, fname):
    if not os.path.exists (fname):
        fname = vmake.yutil.iview (fname, getOriginal = 1)
    #
    # get change lists
    #
    changeLines = os.popen ('p4 changes ' + fname).readlines ()
    bag = Bag ()
    index = 0
    for line in changeLines:
        words = string.split (line)
        submitter = string.split (words [5], '@') [0]
        if timeWeighted:
            count = timeWeigth (words [3])
        elif orderWeighted:
            count = len (changeLines) - index
        else:
            count = 1
        bag.add (submitter, count)
        index = index + 1
    #
    # sort be change count
    #
    sortable = []
    for duser, count in bag.items ():
        sortable.append ((count, duser))
    sortable.sort ()
    sortable.reverse ()
    #
    # translate names
    #
    for count, duser in sortable:
        print "%3d %s" % (count, _duserLookup.get (duser, duser))

def main (options, args):
    for arg in args:
        p4submitters (options.timeWeighted, options.orderWeighted, arg)

def _options ():
    return [
        # (optstring, varname, typechar, default, help)
        ('T', 'timeWeighted', '', None, 'weight by time since change'),
        ('O', 'orderWeighted', '', None, 'weight by submit order'),
        ]

if __name__ == '__main__':
    import optlib
    optlib.optMain2 (main, _options ())
